<?php

class crumbs_Container_CachedLazyData {

  /**
   * @var array
   */
  protected $data = array();

  /**
   * @var crumbs_PluginInfo
   */
  protected $source;

  /**
   * @var array
   */
  protected $keysToCache = array();

  /**
   * @todo Add an interface for $source.
   *   Don't restrict it to crumbs_PluginInfo.
   *
   * @param crumbs_PluginInfo $source
   */
  function __construct($source) {
    $this->source = $source;
    foreach ($source->keysToCache() as $key) {
      $this->keysToCache[$key] = TRUE;
    }
  }

  /**
   * Flush cached data.
   */
  function flushCaches() {
    $this->data = array();
    foreach ($this->keysToCache as $key => $true) {
      cache_clear_all("crumbs:$key", 'cache');
    }
  }

  /**
   * @param string $key
   * @return mixed
   * @throws Exception
   */
  function __get($key) {
    if (!isset($this->data[$key])) {
      if (!empty($this->keysToCache[$key])) {
        $cache = cache_get("crumbs:$key");
        if (isset($cache->data)) {
          return $this->data[$key] = $cache->data;
        }
        if (!method_exists($this->source, $key)) {
          throw new Exception("Key $key not supported.");
        }
        $result = $this->source->$key($this);
        $this->data[$key] = isset($result) ? $result : FALSE;
        cache_set("crumbs:$key", $this->data[$key]);
      }
      else {
        if (!method_exists($this->source, $key)) {
          throw new Exception("Key $key not supported.");
        }
        $result = $this->source->$key($this);
        $this->data[$key] = isset($result) ? $result : FALSE;
      }
    }
    return $this->data[$key];
  }

  function __call($method, $args) {

    if (!isset($this->data[$method])) {
      $this->data[$method] = array();
      if (!empty($this->keysToCache[$method])) {
        $cache = cache_get("crumbs:$method");
        if (isset($cache->data) && is_array($cache->data)) {
          $this->data[$method] = $cache->data;
        }
      }
    }

    switch (count($args)) {

      case 1:
        if (!isset($this->data[$method][$args[0]])) {
          if (!method_exists($this->source, $method)) {
            throw new Exception("Method $method not supported.");
          }
          $result = $this->source->$method($this, $args[0]);
          $this->data[$method][$args[0]] = isset($result) ? $result : NULL;
          if (!empty($this->keysToCache[$method])) {
            cache_set("crumbs:$method", $this->data[$method]);
          }
        }
        return $this->data[$method][$args[0]];

      case 2:
        if (!isset($this->data[$method][$args[0]][$args[1]])) {
          if (!method_exists($this->source, $method)) {
            throw new Exception("Method $method not supported.");
          }
          $result = $this->source->$method($this, $args[0], $args[1]);
          $this->data[$method][$args[0]][$args[1]] = isset($result) ? $result : NULL;
          if (!empty($this->keysToCache[$method])) {
            cache_set("crumbs:$method", $this->data[$method]);
          }
        }
        return $this->data[$method][$args[0]][$args[1]];

      default:
        throw new Exception("Number of arguments not supported: " . count($args));
    }
  }
}
